import type { DictionaryDefinitionPreferred } from '@cspell/cspell-types';
import { SpellingDictionary } from './SpellingDictionary';
export declare type LoadOptions = DictionaryDefinitionPreferred;
interface CacheEntry {
    uri: string;
    options: LoadOptions;
    ts: number;
    state: Promise<Stats | Error>;
    dictionary: Promise<SpellingDictionary>;
}
export declare type LoaderType = keyof Loaders;
export declare type Loader = (filename: string, options: LoadOptions) => Promise<SpellingDictionary>;
export interface Loaders {
    S: Loader;
    C: Loader;
    T: Loader;
    W: Loader;
    default: Loader;
}
export declare function loadDictionary(uri: string, options: DictionaryDefinitionPreferred): Promise<SpellingDictionary>;
/**
 * Check to see if any of the cached dictionaries have changed. If one has changed, reload it.
 * @param maxAge - Only check the dictionary if it has been at least `maxAge` ms since the last check.
 * @param now - optional timestamp representing now. (Mostly used in testing)
 */
export declare function refreshCacheEntries(maxAge?: number, now?: number): Promise<void>;
declare function refreshEntry(entry: CacheEntry, maxAge: number, now: number): Promise<void>;
declare function loadEntry(uri: string, options: LoadOptions, now?: number): CacheEntry;
declare function load(uri: string, options: LoadOptions): Promise<SpellingDictionary>;
export declare const testing: {
    dictionaryCache: Map<string, CacheEntry>;
    refreshEntry: typeof refreshEntry;
    loadEntry: typeof loadEntry;
    load: typeof load;
};
/**
 * Copied from the Node definition to avoid a dependency upon a specific version of Node
 */
interface StatsBase<T> {
    isFile(): boolean;
    isDirectory(): boolean;
    isBlockDevice(): boolean;
    isCharacterDevice(): boolean;
    isSymbolicLink(): boolean;
    isFIFO(): boolean;
    isSocket(): boolean;
    dev: T;
    ino: T;
    mode: T;
    nlink: T;
    uid: T;
    gid: T;
    rdev: T;
    size: T;
    blksize: T;
    blocks: T;
    atimeMs: T;
    mtimeMs: T;
    ctimeMs: T;
    birthtimeMs: T;
    atime: Date;
    mtime: Date;
    ctime: Date;
    birthtime: Date;
}
export declare type Stats = StatsBase<number>;
export {};
//# sourceMappingURL=DictionaryLoader.d.ts.map